import type { LanguageRegistration, ThemedToken, ThemeRegistrationAny } from '@shikijs/types'
import { createHighlighterCore } from '@shikijs/core'
import { createJavaScriptRegexEngine } from '@shikijs/engine-javascript'
import React, { type FunctionComponent } from 'react'
import { ActivityIndicator, ScrollView, type ScrollViewProps, type StyleProp, Text, type TextStyle, View, type ViewStyle } from 'react-native'

export interface SyntaxHighlighterProps {
  text: string
  language: string | LanguageRegistration
  theme: ThemeRegistrationAny
  fontSize?: number
  textStyle?: StyleProp<TextStyle>
  scrollViewProps?: ScrollViewProps
  containerStyle?: StyleProp<ViewStyle>
}

let highlighterInstance: Awaited<ReturnType<typeof createHighlighterCore>> | null = null

async function getHighlighter() {
  if (!highlighterInstance) {
    highlighterInstance = await createHighlighterCore({
      engine: createJavaScriptRegexEngine(),
    })
  }
  return highlighterInstance
}

export const SyntaxHighlighter: FunctionComponent<SyntaxHighlighterProps> = ({ text, language, theme, fontSize = 14, textStyle, scrollViewProps, containerStyle }) => {
  const [tokens, setTokens] = React.useState<ThemedToken[][]>([])
  const [backgroundColor, setBackgroundColor] = React.useState<string>('#0d1117')
  const [loading, setLoading] = React.useState(true)
  const [error, setError] = React.useState<string | null>(null)

  React.useEffect(() => {
    const highlight = async () => {
      try {
        setLoading(true)
        const highlighter = await getHighlighter()

        // Load language and theme
        if (typeof language !== 'string') {
          await highlighter.loadLanguage(language)
        }

        // Ensure theme is loaded
        await highlighter.loadTheme(theme)

        // Get the theme name from the registration
        const themeName = theme.name || Object.getOwnPropertyDescriptor(theme, 'name')?.value || 'github-dark'

        // Highlight the code with explicit theme
        const result = highlighter.codeToTokens(text, {
          lang: typeof language === 'string' ? language : language.name,
          theme: themeName,
        })

        setTokens(result.tokens)
        // Use theme's background color if available
        setBackgroundColor(result.bg || theme.bg || '#0d1117')
        setError(null)
      }
      catch (err) {
        setError(err instanceof Error ? err.message : 'Failed to highlight code')
        console.error('Highlighting error:', err)
      }
      finally {
        setLoading(false)
      }
    }

    highlight()
  }, [text, language, theme])

  const renderToken = (token: ThemedToken, index: number) => {
    const style = {
      color: token.color || '#c9d1d9',
      fontSize,
      ...(token.fontStyle === 1 && { fontStyle: 'italic' as const }),
      ...(token.fontStyle === 2 && { fontWeight: 'bold' as const }),
      ...(token.fontStyle === 4 && { textDecorationLine: 'underline' as const }),
    }

    return (
      <Text key={`${token.offset}-${index}`} style={[textStyle, style]}>
        {token.content}
      </Text>
    )
  }

  const renderLine = (lineTokens: ThemedToken[], index: number) => (
    <View key={index} style={{ flexDirection: 'row', flexWrap: 'wrap' }}>
      {lineTokens.map((token, tokenIndex) => renderToken(token, tokenIndex))}
    </View>
  )

  if (loading) {
    return (
      <View style={[{ backgroundColor, padding: 10 }, containerStyle]}>
        <ActivityIndicator />
      </View>
    )
  }

  if (error) {
    return (
      <View style={[{ backgroundColor, padding: 10 }, containerStyle]}>
        <Text style={{ color: '#ff6b6b' }}>
          Error:
          {error}
        </Text>
      </View>
    )
  }

  return (
    <ScrollView {...scrollViewProps} horizontal style={[{ backgroundColor }, scrollViewProps?.style]} contentContainerStyle={[containerStyle]}>
      <View onStartShouldSetResponder={() => true}>{tokens.map((lineTokens, index) => renderLine(lineTokens, index))}</View>
    </ScrollView>
  )
}

export default SyntaxHighlighter
